//++
//
// ms_centre GUI processing module (GuiModule.cpp)
//
//  2005, Nick Morgalla
// MorSoft / MorSynth
// June 2005
// V1.1
//
// Using SynthEdit SDK V2.180
// Based on examples provided by:
//  2002, Jeff M Soft und Hardware GmbH, All Rights Reserved
//
// Used to centre (or reset) a control knob, e.g. a fine or coarse tuning knob
//--

//+
// Includes
//-
#include "guimodule.h"
#include "windows.h"
#include <stdio.h>
#include <assert.h>

//+
// I/O pins
//
// Note, these definitions must match those in the audio processing class (Module.cpp)
//-
#define PN_Pos			0											// Animation Position
#define PN_Value		1											// Output value
#define PN_ResetValue	2											// The value which represents the rest state
#define PN_Threshold	3											// Range which represents the 'centre' position
#define PN_ForceReset	4											// Used to force a reset if within threshold

//+
// Callback
//-
GuiModule::GuiModule(seGuiCallback seaudioMaster, void *p_resvd1) : SEGUI_base(seaudioMaster, p_resvd1)
{
}

//+
// Destructor
//-
GuiModule::~GuiModule(void)
{
}

//+
// Initialise
//-
void GuiModule::Initialise(bool loaded_from_file)
{
	SEGUI_base::Initialise(loaded_from_file);

	OnValueChangedBack();											// Set initial switch state based on control position
}

//+
// Pin value changed
//-
void GuiModule::OnGuiPinValueChange(SeGuiPin *p_pin)
{
	//+
	// Establish where the change came from
	//-
	int pinID = p_pin->getIndex();

	if  (pinID == PN_Pos)
	{
		OnValueChanged();											// Centre knob
	}
	else
	{
		OnValueChangedBack();										// Set switch based on knob position
	}
}

void GuiModule::OnValueChanged()
{
	//+
	// Get pin values
	//-
	float ResetValue = getPin(PN_ResetValue)->getValueFloat();
	float Threshold  = getPin(PN_Threshold)->getValueFloat();
	float Value      = getPin(PN_Value)->getValueFloat();
	int   ForceReset = getPin(PN_ForceReset)->getValueInt();

	//+
	// Check to see if the switch is activated
	//
	// Not the use of the nudge below.  This is required when operating in Force Reset mode.
	// When Force Reset activates, Value is set to Reset Value, but the control which is generating
	// that value will not necessarily have moved to the Reset Value position.
	//
	// However, in that state the control thinks it is in the Reset Value position, so setting Value
	// equal to Reset Value will not move the control.
	//
	// In order to force the control to move to the Reset Value position, it must be 'nudged' slightly
	// away and then back again.
	//-
	if (getPin(PN_Pos)->getValueFloat() > 0.0f)
	{
		if (ForceReset == 1)
		{
			getPin(PN_Value)->setValueFloat(ResetValue + 0.0001f);	// Nudge the control
		}
		
		getPin(PN_Value)->setValueFloat(ResetValue);				// Reset the control
	}
	else
	{
		if ((Value >= ResetValue - Threshold) && (Value <= ResetValue + Threshold))
		{
			getPin(PN_Pos)->setValueFloat(1.0f);					// Keep the switch activated if knob is within threshold
		}
	}
}

void GuiModule::OnValueChangedBack()
{
	//+
	// Get pin values
	//-
	float ResetValue = getPin(PN_ResetValue)->getValueFloat();
	float Threshold  = getPin(PN_Threshold)->getValueFloat();
	float Value      = getPin(PN_Value)->getValueFloat();
	int   ForceReset = getPin(PN_ForceReset)->getValueInt();

	//+
	// Check to see if the control is within threshold of reset value
	//-
	if ((Value >= ResetValue - Threshold) && (Value <= ResetValue + Threshold))
	{
		//+
		// Do we force a reset?
		//-
		if (ForceReset == 1)  
		{
			getPin(PN_Value)->setValueFloat(ResetValue);			// Reset the control
		}

		getPin(PN_Pos)->setValueFloat(1.0f);						// Keep the switch activated
	}
	else
	{
		getPin(PN_Pos)->setValueFloat(0.0f);						// Release the switch
	}
}
