//++
//
// ms_centre audio processing module (Module.cpp)
//
//  2005, Nick Morgalla
// MorSoft / MorSynth
// June 2005
// V1.1
//
// Using SynthEdit SDK V2.180
// Based on examples provided by:
//  2002, Jeff M Soft und Hardware GmbH, All Rights Reserved
//
//--

//++
//
// Abstract
//
// Centre (reset) knob
//
// Main logic is in the GuiModule
//
//--

//+
// Includes
//-
#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>

//+
// I/O pins
//-
#define PN_Pos			0									// Animation Position
#define PN_Value		1									// Output value
#define PN_ResetValue	2									// The value which represents the rest state
#define PN_Threshold	3									// Range which represnet the 'centre' position
#define PN_ForceReset	4									// Used to force a reset if within threshold

//+
// Callback function
//-
Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

//+
// Destructor
//-
Module::~Module()
{
}

//+
// Perform initialisation
//-
void Module::open()
{
	//+
	// Because this is a gui only module, no audio processing is required
	//-
	SET_PROCESS_FUNC(Module::sub_process_static);									// Set active processing module
	SEModule_base::open();															// Call the base class
}

//+
// No processing to do so put module to sleep on audio side
//-
void Module::sub_process_static(long buffer_offset, long sampleFrames )
{
	//+
	// Since this is a gui module, no audio processing to do
	//-
	CallHost(seaudioMasterSleepMode);
}

//+
// Module description
//-
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	properties->gui_flags = CF_STRUCTURE_VIEW;

	//+
	// Module name (as viewed by the user)
	//-
	properties->name = "Knob Centre";

	//+
	// Return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug. No two plugs may have the same id.
	//-
	properties->id = "MorSynth Centre";

	//+
	// Info, displayed when 'Help' is activated
	//-
	properties->about = " Nick Morgalla 2005\n\n"
						"MorSynth Centre V1.1\n\n"
						"Enables a switch (e.g. Bitmap Image) to centre or reset a knob\n\n"
						"Value = Reset Value if the switch (animation position > 0) is activated\n\n"
						"Also, the switch is activated (animation position = 1) if Value is within\n"
						"+ or - Threshold of Reset Value\n\n"
						"If Force Reset = On, then whenever value is within Threshold of Reset Value\n"
						"Value is forced to equal Reset Value";

	return true;
}

//+
// Describe the pins (plugs)
//-
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch (index)
	{
	case PN_Pos:
		properties->name				= "Animation Position";
		properties->direction			= DR_IN;
		properties->datatype			= DT_FLOAT;
		properties->flags				= IO_UI_COMMUNICATION;
		break;
	case PN_Value:
		properties->name				= "Value";
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FLOAT;
		properties->flags				= IO_UI_COMMUNICATION;
		break;
	case PN_ResetValue:
		properties->name				= "Reset Value";
		properties->direction			= DR_IN;
		properties->datatype			= DT_FLOAT;
		properties->default_value		= "0";
		properties->flags				= IO_UI_COMMUNICATION;
		break;
	case PN_Threshold:
		properties->name				= "Threshold";
		properties->direction			= DR_IN;
		properties->datatype			= DT_FLOAT;
		properties->default_value		= "0.3";
		properties->flags				= IO_UI_COMMUNICATION;
		break;
	case PN_ForceReset:
		properties->name				= "Force Reset";
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		properties->datatype_extra		= "No=0,Yes=1";
		properties->default_value		= "1";
		properties->flags				= IO_UI_COMMUNICATION;
		break;	
	default:
		return false;
	};

	return true;
}
